/**
 * <PRE>
 * Copyright and Permissions:
 *  Copyright Geoff Dale 1996, All rights reserved.
 *  You may freely redistribute this file as long as no modifications are made
 *     to the code or the comments (including this copyright).
 *  You may freely redistribute the compiled .class file created by this file.
 *  You may freely subclass from this class.
 *
 *
 * Naming Conventions:
 *  ClassName
 *  methodName
 *  void setVariableName( type a )  - setter accessor method
 *  type getVariableName()          - getter accessor method
 *  VARIABLE_NAME       - constants (static final)
 *  _VariableName       - (private or protected) instance variable
 *  s_VariableName      - static instance variable
 *  a_VariableName      - method argument
 *  a                   - setter argument
 *  variableName        - public instance or local variable
 *
 * History:
 *  gdale   09/30/96 - created.
 *
 * </PRE>
 * @author Geoff Dale <geoff1@home.com>
 * @version 0.1
 */


import java.io.*;
import java.util.Properties;
import java.awt.Component;

/**
 * <PRE>
 * A version of FilterInputStream that we can create by name. Usually used as a
 *  base class for drop in filters. You can use the Class Method newFilter to
 *  instantiate the input stream.
 *
 * ex:
 * </PRE>
 */
public class DynamicFilterInputStream
    extends     FilterInputStream
    implements  DynamicPropertyInterface
{
//
// -- Instance Variables --
//
    protected Properties _Properties;

//
// -- Constructor --
//
    /**
     * No arg constructor so that we may be created by newInstance.
     */
    public DynamicFilterInputStream()
    {
        super( null );
    }

//
// -- Initializer --
//
    /**
     * Initialize before use.
     *
     * @param InputStream stream to be filtered
     */
    public void init( InputStream a_InputStreamToBeFiltered )
    {
        in = a_InputStreamToBeFiltered;
            // in is inherited from FilterInputStream
    }

//
// -- Class Methods --
//
    /**
     * Create a new instance of a designated filter.
     *
     * @param       InputStream stream to be filtered.
     * @param       String classname of filter to be created.
     * @return      A DynamicFilterInputStream subclassed object
     * @exception   InstantiationException If you try to instantiate  an
     *                  abstract class or an interface, or if the
     *                  instantiation fails for some other reason.
     * @exception   IllegalAccessException If the class or initializer
     *                  is not accessible.
     * @exception   ClassNotFoundException If the designated class could not
     *                  be found.
     * @exception   ClassCastException if String arg does not represent a
     *                  subclass.
     */
    public static DynamicFilterInputStream newFilter(
        String          a_DynamicFilterClassName,
        InputStream     a_InputStreamToBeFiltered )
        throws          ClassNotFoundException,
                        InstantiationException,
                        IllegalAccessException
    {
        Class inputStreamClass = Class.forName( a_DynamicFilterClassName );
        DynamicFilterInputStream inputStream =
            (DynamicFilterInputStream)inputStreamClass.newInstance();

        inputStream.init( a_InputStreamToBeFiltered );

        return inputStream;
    }

//
// -- Public Methods --
//
    /**
     * @return true if object actually supports the property
     */
    public boolean setProperty( String a_Key, String a_Value )
    {
        if ( _Properties == null )
        { newProperties(); }

        _Properties.put( (Object) a_Key,  (Object) a_Value );

        return false;
    }

    /**
     * @return the property; null if property doesn't exist
     */
    public String getProperty( String a_Key )
    {
        String property = null;

        if ( _Properties != null )
        {
            property = _Properties.getProperty( a_Key );
        }

        return property;
    }

    /**
     * @return Two dimensional array of property info;
     *          null if no properties are supported.
     *         <BR> String[numKeys][3]
     *         <BR> String[i][0] is a key
     *         <BR> String[i][1] is the information,
     *              describing the property's use
     *         <BR> String[i][2] is one byte; flags defined by
     *              DynamicPropertyConstants.PROPERTIES_FLAG_ flags.
     * @see DynamicPropertyConstants
     */
    public String[][] getPropertyInfo(){ return null; }

    /**
     * @return Array of property keys supported
     *          null if no properties are supported.
     *         <BR> String[] = propertyKey
     */
    public String[] getPropertyKeys(){ return null; }

    /**
     * List properties, for debugging
     */
    public void list( PrintStream a_PrintStream )
    {
        if ( _Properties != null )
        {
            _Properties.list( a_PrintStream );
        }
    }

//
// -- Protected Methods --
//
    protected void newProperties()
    {
        _Properties = new Properties();
    }


}